require 'csv'
require 'stringio'
require 'rexml/document'
require 'rexml/parsers/pullparser'

#check_windows
def check_windows
  if RUBY_PLATFORM.downcase =~ /mswin(?!ce)|mingw|cygwin|bccwin/
    return 1
  else
    return 0
  end
end

# Create CSV Data
def create_csv_data(ary)
  io = StringIO.new("", "r+")
  writer = CSV::Writer.generate(io, ',')

  header = ["Process Group Name", "Calendar Name", "CALENDAR_BIGIN", "CALENDAR_END", "Process Definition Name"]
  writer.add_row(header)

  ary.each{|row|
    writer.add_row(row)
  }

  writer.close
  io.rewind

  str = io.read

  return str
end

### Check Use business calendar
def biz_cal_use_check(xpdl_filename, dirname = "")
  # Define
  pd_Name_Element_Data = "//WorkflowProcesses"
  workflowProcess_Data = "WorkflowProcess"
  search_Element_Data = "//ibpm:TimerDefSet"
  timer_Element_Data = "TimerDef"
  duedate_Element_Data = "DUEDATETIMERDEF"
  search_Element_Field_Data = "//DataField"
  initialValue_Data = "InitialValue"
  uda_BizCal_Name = "__businessCalendar"

  # init
  pd_name = ""
  hitflag = false
  biz_cal_ary = []
  datafield_name_ary = []
  default_cal_name = "Default"

  # REXML
  begin
    doc = REXML::Document.new(File.open(xpdl_filename))
  rescue => exc
    STDERR.print "File Access Error. #{dirname.gsub("\\","/")}#{xpdl_filename.gsub("\\","/")}\n"
    exit 1
  end

  begin
    # Get Process Definition Name
    doc.each_element(pd_Name_Element_Data) { |workflow|
      pd_name = workflow.elements[workflowProcess_Data].attributes["Name"]
    }
    
    # Search TimerDef
    doc.each_element(search_Element_Data) { |timerdef|
      # Timer
      if timerdef.elements[timer_Element_Data] != nil
        timerdef.elements.each(timer_Element_Data){ |timerdef_item|
          type = timerdef_item.attributes["type"]
          if type == "3" || type == "4"
            hitflag = true
            datafield_name_ary << timerdef_item.attributes["time"]
          end
        }
      end
      # Duedate
      if timerdef.elements[duedate_Element_Data] != nil
        timerdef.elements.each(duedate_Element_Data){ |timerdef_item|
          type = timerdef_item.attributes["type"]
          if type == "4"
            hitflag = true
            datafield_name_ary << timerdef_item.attributes["time"]
          end
        }
      end
    }
    
    # Search User BizCal
    if hitflag == true
      # Get Default Cal of XPDL
      doc.each_element(search_Element_Field_Data) { |datafield|
        if datafield.attributes["Name"] == uda_BizCal_Name
          if datafield.elements[initialValue_Data] != nil
            tmp_cal_name = datafield.elements[initialValue_Data].text
            if tmp_cal_name != nil && tmp_cal_name != ""
              # Set UDA BizCal
              default_cal_name = datafield.elements[initialValue_Data].text
              break
            end
          end
        end
      }
      # Get User BizCal
      doc.each_element(search_Element_Field_Data) { |datafield|
        if datafield_name_ary.find{ |item| item == datafield.attributes["Name"] } != nil
          chk_value = datafield.elements[initialValue_Data].text
          if chk_value =~ /UC\((.+?)\)/
            # User BizCal
            biz_cal_ary << $1
          else
            # Default Cal or UDA BizCal
            biz_cal_ary << default_cal_name
          end
        end
      }
    end
  rescue => exc
    # Item is None. Continue.
  end
  
  return hitflag, pd_name, biz_cal_ary
end

### Get Calendar BEGIN/END
def get_cal_begin_end(cal_filename, dirname = "")
  # Define
  def_Begin = "CALENDAR_BEGIN"
  def_End = "CALENDAR_END"

  # init
  begin_str = ""
  end_str = ""

  # Search
  begin
    file = File.open(cal_filename,"r")
  rescue => exc
    STDERR.print "File Access Error. #{dirname.gsub("\\","/")}#{cal_filename.gsub("\\","/")}\n"
    exit 1
  end
  
  begin
    file.each{|line|
      line_tmp = line.chomp
      item_tmp = line_tmp.split("=",2)
      
      # get CALENDAR_BEGIN
      if item_tmp[0] == def_Begin
        begin_str = item_tmp[1].gsub(";","")
      end
      
      # get CALENDAR_END
      if item_tmp[0] == def_End
        end_str = item_tmp[1].gsub(";","")
      end
    }
  rescue => exc
    STDERR.print "Internal Error. Detail=#{exc.to_s}\n"
    begin
      file.close
    rescue
    end
    exit 1
  end
  
  begin
    file.close
  rescue
  end
  
  return begin_str, end_str
end

### MAIN ###
Dir_path={
  "windows"=>["#{ENV['SWRBA_HOME']}/../IBPM/server/instance/default/resources/Default.cal",
              "#{ENV['SWRBA_HOME']}/etc/share/data/IBPMShared/tenants/Default/attachments/apps"],
  "linux"=>["/opt/FJSVibpm/server/instance/default/resources/Default.cal",
            "/opt/FJSVswrbam/data/IBPMShared/tenants/Default/attachments/apps"]
}

begin
  # check platform
  if check_windows == 1
    platform = "windows"
  else
    platform =  "linux"
  end

  # ARGV check
  if ARGV[0] == nil || ARGV[0] == ""
    csvflag = false
  elsif ARGV[0] == "-csv"
    csvflag = true
  else
    STDERR.print "Argument error. #{ARGV[0]}\n"
    exit 1
  end

  # Get file list
  tmp_dirpath = Dir_path[platform]
  defaultCal = tmp_dirpath[0] # Default Calendar
  pg_dirname = tmp_dirpath[1] # Process Group Dir

  # Set output array
  out_ary = []

  # Default Calendar
  defcal_begin_str, defcal_end_str = get_cal_begin_end(defaultCal)
  out_ary << ["System", "Default", defcal_begin_str, defcal_end_str,""]

  # Get Process Group List
  pg_list_ary = Dir.entries(pg_dirname)
  pg_list_ary.sort!{|a, b| a.downcase <=> b.downcase }

  # Loop Process Group Dir list
  pg_list_ary.each{ |item_name|
    next if item_name == "." || item_name == ".."
    dirname = "#{pg_dirname}/#{item_name}"
    next if File.directory?(dirname) == false
    pg_name = dirname.split("/")[-1]
    next if pg_name == "System"
    
    # xpdl file Loop
    pwd = Dir.getwd
    Dir.chdir(dirname)
    xpdl_hash = {}
    xpdl_file_ary = Dir.glob("*.xpdl")
    xpdl_file_ary.sort!{|a, b| a.downcase <=> b.downcase }
    xpdl_file_ary.each{ |xpdl_filename|
      pd_name = ""
      hitflag = false
      hitflag, pd_name, biz_cal_ary = biz_cal_use_check(xpdl_filename, dirname + "/")
      if hitflag == true
        xpdl_hash[pd_name] = biz_cal_ary
      end
    }
    
    # cal file Loop
    cal_file_ary = Dir.glob("*.cal")
    cal_file_ary.sort!{|a, b| a.downcase <=> b.downcase }
    cal_file_ary.each{ |cal_filename|
      pd_name = ""
      cal_name = cal_filename.split("/")[-1].split(".")[0]
      begin_str, end_str = get_cal_begin_end(cal_filename, dirname + "/")
      tmp_pd_name_ary = []
      xpdl_hash.each{ |key, value_ary|
        if value_ary.find{ |item| item == cal_name } != nil
          # Bizcal that uses process difinition is checked
          tmp_pd_name_ary << key
        end
      }
      tmp_pd_name_ary.uniq!
      tmp_pd_name_ary.sort!{|a, b| a.downcase <=> b.downcase }
      tmp_pd_name_ary.each{ |item|
        if pd_name == ""
          pd_name = item.dup
        else
          pd_name.concat(",#{item}")
        end
      }
      # Set out_ary
      out_ary << [pg_name, cal_name, begin_str, end_str, pd_name]
    }
    Dir.chdir(pwd)
  }

  # Stdout print
  if csvflag == true
    # Print CSV
    out_data = create_csv_data(out_ary)
    STDOUT.print out_data
  else
    # Print List
    before_pg_name = ""
    print_flag = false
    out_ary.each{ |ary|
      if before_pg_name != ary[0]
        STDOUT.print "-------------------------------------------\n"
        STDOUT.print "Process Group Name = #{ary[0]}\n"
        before_pg_name = ary[0]
      end
      STDOUT.print "  #{ary[1]}\n"
      STDOUT.print "    CALENDAR_BEGIN=#{ary[2]}\n"
      STDOUT.print "    CALENDAR_END=#{ary[3]}\n"
      pd_ary = CSV.parse(ary[4])
      pd_ary = pd_ary[0]
      if pd_ary != nil
        pd_ary.each{ |tmp_pdname|
          STDOUT.print "      #{tmp_pdname}\n"
        }
      end
      print_flag = true
    }
    if print_flag == true
      STDOUT.print "-------------------------------------------\n"
    end
  end
rescue => exc
  STDERR.print "Internal Error. Detail=#{exc.to_s}\n"
  exit 1
end

exit 0
