require 'csv'
require 'date'
require 'fileutils'

### Replace File ###
def replace_file(filename, begin_date, end_date, dirname, backup_dirname, ignore_flag)
  # Read file
  begin
    buffer = File.open(filename, "r"){ |f| f.read() }
  rescue => exc
    if ignore_flag == false
      STDERR.print "File Access Error. #{dirname.gsub("\\","/")}#{filename.gsub("\\","/")}\n"
      exit 1
    end
  end

  # Backup
  if ignore_flag == false
    begin
      Dir.mkdir(backup_dirname, File.umask | 0777) if File.directory?(backup_dirname) == false
      bk_filename = filename.split("/")[-1]
      if File.exist?("#{backup_dirname}/#{bk_filename}") == false
        File.open("#{backup_dirname}/#{bk_filename}", "w"){ |f| f.write(buffer) }
      end
    rescue => exc
      STDERR.print "File Access Error. #{backup_dirname.gsub("\\","/")}/#{bk_filename}\n"
      exit 1
    end
  end

  begin
    # CALENDAR_BEGIN Replace
    buffer.gsub!(/CALENDAR_BEGIN=.*;/, "CALENDAR_BEGIN=#{begin_date};")
    # CALENDAR_END Replace
    buffer.gsub!(/CALENDAR_END=.*;/, "CALENDAR_END=#{end_date};")
  rescue => exc
    if ignore_flag == false
      STDERR.print "Internal Error. Detail=#{exc.to_s}\n"
      exit 1
    end
  end

  # Write File
  begin
    File.open(filename, "w"){ |f| f.write(buffer) }
  rescue => exc
    if ignore_flag == false
      STDERR.print "File Access Error. #{dirname.gsub("\\","/")}#{filename.gsub("\\","/")}\n"
      # Recovery
      begin
        FileUtils.copy("#{backup_dirname}/#{bk_filename}", filename)
      rescue => exc2
        STDERR.print "Recovery Error. #{dirname.gsub("\\","/")}#{filename.gsub("\\","/")}\n"
      end
      exit 1
    end
  end
  
  return
end

### MAIN ###
defaultCal1 = "#{ENV['SWRBA_STUDIO_HOME']}/ibpm/Data/calendar/Default.cal"
defaultCal2 = "#{ENV['SWRBA_STUDIO_HOME']}/ibpm/template/Default.cal"

begin 
  # check argv
  cmd_dirname = ARGV[0]

  if ARGV[1] == nil || ARGV[1] == ""
    STDERR.print "Argument error.\n"
    exit 1
  end

  workspaceFolder = ARGV[1]
  if File.directory?(workspaceFolder) == false
    STDERR.print "Argument error. #{workspaceFolder}\n"
    exit 1
  end

  if ARGV[2] == nil || ARGV[3] == nil
    STDERR.print "Argument error.\n"
    exit 1
  end

  begin_str = ARGV[2]
  end_str = ARGV[3]

  if begin_str == "" || end_str == ""
    STDERR.print "Argument error.\n"
    exit 1
  end

  if begin_str !~ /^\d{4}\/\d{2}\/\d{2}$/
    STDERR.print "Invalid argument. #{begin_str}\n"
    exit 1
  end

  if end_str !~ /^\d{4}\/\d{2}\/\d{2}$/
    STDERR.print "Invalid argument. #{end_str}\n"
    exit 1
  end

  begin
    begin_date = Date.parse(begin_str)
  rescue => exc
    STDERR.print "Invalid argument. #{begin_str}\n"
    exit 1
  end

  begin
    end_date = Date.parse(end_str)
  rescue => exc
    STDERR.print "Invalid argument. #{end_str}\n"
    exit 1
  end

  # CALENDAR_BEGIN + 10years - 1day
  check_date = (begin_date >> (12*10)) - 1
  # check_date is small : Over 10years
  if (check_date <=> end_date) == -1
    STDERR.print "The calendar duration exceeds the maximum allowed limit of 10 years.\n"
    exit 1
  end

  backup_dirname = "#{cmd_dirname}../backup/studio"

  # Replace Default Calendar
  sys_backup_dirname = "#{backup_dirname}/System"
  replace_file(defaultCal1, begin_str, end_str, "", "", true)
  replace_file(defaultCal2, begin_str, end_str, "", sys_backup_dirname, false)
  STDOUT.print "System/Default.cal\n"

  # Get Process Group List
  pg_list_ary = Dir.entries(workspaceFolder)
  pg_list_ary.sort!{|a, b| a.downcase <=> b.downcase }

  # Replace Process Group Calendar
  pg_list_ary.each{ |item_name|
    next if item_name == "." || item_name == ".."
    dirname = "#{workspaceFolder}/#{item_name}"
    next if File.directory?(dirname) == false
    
    pwd = Dir.getwd
    Dir.chdir(dirname)
    pg_name = item_name
    filelist_ary = Dir.glob("*.cal")
    filelist_ary.sort!{|a, b| a.downcase <=> b.downcase }
    filelist_ary.each{|file|
      cal_name = file
      pg_backup_dirname = "#{backup_dirname}/#{pg_name}"
      replace_file(file, begin_str, end_str, dirname + "/", pg_backup_dirname, false)
      STDOUT.print "#{pg_name}/#{cal_name}\n"
    }
    Dir.chdir(pwd)
  }
  
rescue => exc
  STDERR.print "Internal Error. Detail=#{exc.to_s}\n"
  exit 1
end

exit 0
