require 'csv'
require 'date'
require 'fileutils'

### Check_windows ###
def check_windows
  if RUBY_PLATFORM.downcase =~ /mswin(?!ce)|mingw|cygwin|bccwin/
    return 1
  else
    return 0
  end
end

### Replace File ###
def replace_file(filename, begin_date, end_date, dirname, backup_dirname, ignore_flag)
  # Read file
  begin
    buffer = File.open(filename, "r"){ |f| f.read() }
  rescue => exc
    if ignore_flag == false
      STDERR.print "File Access Error. #{dirname.gsub("\\","/")}#{filename.gsub("\\","/")}\n"
      exit 1
    end
  end

  # Backup
  if ignore_flag == false
    begin
      Dir.mkdir(backup_dirname, File.umask | 0777) if File.directory?(backup_dirname) == false
      bk_filename = filename.split("/")[-1]
      if File.exist?("#{backup_dirname}/#{bk_filename}") == false
        File.open("#{backup_dirname}/#{bk_filename}", "w"){ |f| f.write(buffer) }
      end
    rescue => exc
      STDERR.print "File Access Error. #{backup_dirname.gsub("\\","/")}/#{bk_filename}\n"
      exit 1
    end
  end

  begin
    # CALENDAR_BEGIN Replace
    buffer.gsub!(/CALENDAR_BEGIN=.*;/, "CALENDAR_BEGIN=#{begin_date};")
    # CALENDAR_END Replace
    buffer.gsub!(/CALENDAR_END=.*;/, "CALENDAR_END=#{end_date};")
  rescue => exc
    if ignore_flag == false
      STDERR.print "Internal Error. Detail=#{exc.to_s}\n"
      exit 1
    end
  end

  # Write File
  begin
    File.open(filename, "w"){ |f| f.write(buffer) }
  rescue => exc
    if ignore_flag == false
      STDERR.print "File Access Error. #{dirname.gsub("\\","/")}#{filename.gsub("\\","/")}\n"
      # Recovery
      begin
        FileUtils.copy("#{backup_dirname}/#{bk_filename}", filename)
      rescue => exc2
        STDERR.print "Recovery Error. #{dirname.gsub("\\","/")}#{filename.gsub("\\","/")}\n"
      end
      exit 1
    end
  end
  
  return
end

### MAIN ###
Dir_path={
  "windows"=>["#{ENV['SWRBA_HOME']}/../IBPM/server/instance/default/resources/Default.cal",
              "#{ENV['SWRBA_HOME']}/../IBPM/server/instance/default/tenants/Default/attachments/apps/System/Default.cal",
              "#{ENV['SWRBA_HOME']}/../IAPS/F3FMisjee/var/domains/interstage/applications/j2ee-apps/InterstageBPMConsole/console_war/tenants/default/apps",
              "#{ENV['SWRBA_HOME']}/etc/share/data/IBPMShared/tenants/Default/attachments/apps"],
  "linux"=>["/opt/FJSVibpm/server/instance/default/resources/Default.cal",
            "/opt/FJSVibpm/server/instance/default/tenants/Default/attachments/apps/System/Default.cal",
            "/var/opt/FJSVisjee/domains/interstage/applications/j2ee-apps/InterstageBPMConsole/console_war/tenants/default/apps",
            "/opt/FJSVswrbam/data/IBPMShared/tenants/Default/attachments/apps"]
}

begin 
  # check platform
  if check_windows == 1
    platform = "windows"
  else
    platform =  "linux"
  end

  # check argv
  cmd_dirname = ARGV[0]

  if ARGV[1] == nil || ARGV[2] == nil
    STDERR.print "Argument error.\n"
    exit 1
  end

  begin_str = ARGV[1]
  end_str = ARGV[2]

  if begin_str == "" || end_str == ""
    STDERR.print "Argument error.\n"
    exit 1
  end

  if begin_str !~ /^\d{4}\/\d{2}\/\d{2}$/
    STDERR.print "Invalid argument. #{begin_str}\n"
    exit 1
  end

  if end_str !~ /^\d{4}\/\d{2}\/\d{2}$/
    STDERR.print "Invalid argument. #{end_str}\n"
    exit 1
  end

  begin
    begin_date = Date.parse(begin_str)
  rescue => exc
    STDERR.print "Invalid argument. #{begin_str}\n"
    exit 1
  end

  begin
    end_date = Date.parse(end_str)
  rescue => exc
    STDERR.print "Invalid argument. #{end_str}\n"
    exit 1
  end

  # CALENDAR_BEGIN + 10years - 1day
  check_date = (begin_date >> (12*10)) - 1
  # check_date is small : Over 10years
  if (check_date <=> end_date) == -1
    STDERR.print "The calendar duration exceeds the maximum allowed limit of 10 years.\n"
    exit 1
  end

  backup_dirname = "#{cmd_dirname}../backup/manager"

  # Get file list
  tmp_dirpath = Dir_path[platform]
  defaultCal1 = tmp_dirpath[0] # Default Calendar1
  defaultCal2 = tmp_dirpath[1] # Default Calendar2
  pg_dirname1 = tmp_dirpath[2] # Process Group Dir1
  pg_dirname2 = tmp_dirpath[3] # Process Group Dir2

  # Replace Default Calendar
  sys_backup_dirname = "#{backup_dirname}/System"
  replace_file(defaultCal1, begin_str, end_str, "", sys_backup_dirname, false)
  replace_file(defaultCal2, begin_str, end_str, "", "", true)
  STDOUT.print "System/Default.cal\n"

  # Get Process Group List
  pg_list_ary = Dir.entries(pg_dirname1)
  pg_list_ary.sort!{|a, b| a.downcase <=> b.downcase }

  # Replace Process Group Calendar
  pg_list_ary.each{ |item_name|
    next if item_name == "." || item_name == ".."
    dirname = "#{pg_dirname1}/#{item_name}"
    next if File.directory?(dirname) == false
    
    pwd = Dir.getwd
    Dir.chdir(dirname)
    filelist_ary1 = Dir.glob("*.cal")
    filelist_ary1.sort!{|a, b| a.downcase <=> b.downcase }
    filelist_ary1.each{|file|
      replace_file(file, begin_str, end_str, "", "", true)
    }
    Dir.chdir(pwd)
  }

  # Get Process Group List
  pg_list_ary = Dir.entries(pg_dirname2)
  pg_list_ary.sort!{|a, b| a.downcase <=> b.downcase }

  # Replace Process Group Calendar
  pg_list_ary.each{ |item_name|
    next if item_name == "." || item_name == ".."
    dirname = "#{pg_dirname2}/#{item_name}"
    next if File.directory?(dirname) == false
    
    pwd = Dir.getwd
    Dir.chdir(dirname)
    pg_name = item_name
    filelist_ary2 = Dir.glob("*.cal")
    filelist_ary2.sort!{|a, b| a.downcase <=> b.downcase }
    filelist_ary2.each{|file|
      cal_name = file
      pg_backup_dirname = "#{backup_dirname}/#{pg_name}"
      if pg_name == "System"
        replace_file(file, begin_str, end_str, "", "", true)
      else
        replace_file(file, begin_str, end_str, dirname + "/", pg_backup_dirname, false)
      end
      if pg_name != "System"
        STDOUT.print "#{pg_name}/#{cal_name}\n"
      end
    }
    Dir.chdir(pwd)
  }

rescue => exc
  STDERR.print "Internal Error. Detail=#{exc.to_s}\n"
  exit 1
end

exit 0
